@@ -10,6 +10,7 @@ import math
 from Tkinter import *
 import tkColorChooser
 import tkFileDialog
+import tkFont
 import tkMessageBox
 import tkSimpleDialog
 from PIL import Image, ImageTk # it overrides Tkinter.Image so it must be after Tkinter import statement
@@ -47,6 +48,10 @@ class GameViewer(Canvas):
         Canvas.__init__(self, master, width=800, height=600, bg='black')
         self.pack(expand=YES, fill=BOTH)
 
+        # To enable receiving wheel rolling events under windows, we
+        # need this action before bindings:
+        self.focus_set()
+
         # bindings
         self.bind('<B1-Motion>', self._mouse_motion_callback)
         self.bind('<ButtonRelease-1>', self._release_callback)
@@ -115,6 +120,13 @@ class GameViewer(Canvas):
                 sprite_name = switch[unit.type.main_name](unit)
                 self._draw(sprite_name, position)
 
+                x, y = self._to_screen_coordinate(position)
+                color = '#' + "%02x%02x%02x" % unit.player.color
+                font = self._get_font_for_current_zoom()
+                self.create_text(x, y, fill=color, text=unit.player.name,
+                                 font=font, tags=['text'],
+                                 state=NORMAL if font else HIDDEN)
+
                 if isinstance(unit.action, actions.GatherAction):
                     draw_arrow_from_to(unit.position, unit.action.source)
                 elif isinstance(unit.action, actions.StoreAction):
@@ -152,6 +164,16 @@ class GameViewer(Canvas):
         self.create_image(x, y, image=image, anchor=NW,
                           state=state, tags=[name]+extra_tags)
 
+    def _get_font_for_current_zoom(self):
+        size = int(40*self._zoom)
+        if size < 10:
+            if size >= 6:
+                return tkFont.Font(size=10)
+            else:
+                return None
+        else:
+            return tkFont.Font(size=size)
+
     @memoized
     def _get_image(self, name):
         """ Return (PIL.)Image instance. """
@@ -216,18 +238,24 @@ class GameViewer(Canvas):
             image = self._get_scaled_sprite(name)
             self.itemconfigure(name, image=image)
 
+        # scale all texts
+        font = self._get_font_for_current_zoom()
+        self.itemconfigure('text', font=font,
+                           state = NORMAL if font else HIDDEN)
+
         # move all images
         factor = zoom/old_zoom
         self.scale(ALL, XS, YS, factor, factor)
 
     def _roll_wheel_callback(self, event):
         if self._game:
-            # respond to Linux or Windows wheel event
             delta = 0
-            if event.num == 5 or event.delta == -120:
+            if event.num == 5: # respond Linux wheel event
                 delta -= 1
-            if event.num == 4 or event.delta == 120:
+            elif event.num == 4: # -//-
                 delta += 1
+            else: # respond Windows wheel event
+                delta += event.delta // 120
 
             factor = GameViewer.SCROLLING_SENSITIVITY**delta
             self._set_zoom(self._zoom*factor, (event.x, event.y))
